<?php

/**
 * The file that defines the core plugin class
 *
 * A class definition that includes attributes and functions used across both the
 * public-facing side of the site and the admin area.
 *
 * @link       http://4dealer.ru/
 * @since      1.0.0
 *
 * @package    Wp_Fdeal
 * @subpackage Wp_Fdeal/includes
 */

/**
 * The core plugin class.
 *
 * This is used to define internationalization, admin-specific hooks, and
 * public-facing site hooks.
 *
 * Also maintains the unique identifier of this plugin as well as the current
 * version of the plugin.
 *
 * @since      1.0.0
 * @package    Wp_Fdeal
 * @subpackage Wp_Fdeal/includes
 * @author     Maksim Onokhin <root@cybermax.spb.ru>
 */
class Wp_Fdeal
{

    /**
     * The loader that's responsible for maintaining and registering all hooks that power
     * the plugin.
     *
     * @since    1.0.0
     * @access   protected
     * @var      Wp_Fdeal_Loader $loader Maintains and registers all hooks for the plugin.
     */
    protected $loader;

    /**
     * The unique identifier of this plugin.
     *
     * @since    1.0.0
     * @access   protected
     * @var      string $plugin_name The string used to uniquely identify this plugin.
     */
    protected $plugin_name;

    /**
     * The current version of the plugin.
     *
     * @since    1.0.0
     * @access   protected
     * @var      string $version The current version of the plugin.
     */
    protected $version;

    /**
     * Define the core functionality of the plugin.
     *
     * Set the plugin name and the plugin version that can be used throughout the plugin.
     * Load the dependencies, define the locale, and set the hooks for the admin area and
     * the public-facing side of the site.
     *
     * @since    1.0.0
     */


    public function __construct()
    {

        $this->plugin_name = 'wp-fdeal';
        $this->version = '1.5.1';
        $this->localActualVersionFile = plugin_dir_path(dirname(__FILE__)) . 'admin/wp_version.txt';

        /**
         * Проверка текущей версии - если версия старая - показываем уведомление
         */

        $this->remoteUpdatePluginVersion();

        $this->load_dependencies();
        $this->set_locale();
        $this->define_admin_hooks();
        $this->define_public_hooks();

        global $wpdb;

        $this->db = $wpdb;
    }

    /**
     * Load the required dependencies for this plugin.
     *
     * Include the following files that make up the plugin:
     *
     * - Wp_Fdeal_Loader. Orchestrates the hooks of the plugin.
     * - Wp_Fdeal_i18n. Defines internationalization functionality.
     * - Wp_Fdeal_Admin. Defines all hooks for the admin area.
     * - Wp_Fdeal_Public. Defines all hooks for the public side of the site.
     *
     * Create an instance of the loader which will be used to register the hooks
     * with WordPress.
     *
     * @since    1.0.0
     * @access   private
     */
    private function load_dependencies()
    {

        /**
         * The class responsible for orchestrating the actions and filters of the
         * core plugin.
         */
        require_once plugin_dir_path(dirname(__FILE__)) . 'includes/class-wp-fdeal-loader.php';

        /**
         * The class responsible for defining internationalization functionality
         * of the plugin.
         */
        require_once plugin_dir_path(dirname(__FILE__)) . 'includes/class-wp-fdeal-i18n.php';

        /**
         * The class responsible for defining all actions that occur in the admin area.
         */
        require_once plugin_dir_path(dirname(__FILE__)) . 'admin/class-wp-fdeal-admin.php';

        /**
         * The class responsible for defining all actions that occur in the public-facing
         * side of the site.
         */
        require_once plugin_dir_path(dirname(__FILE__)) . 'public/class-wp-fdeal-public.php';

        require_once plugin_dir_path(dirname(__FILE__)) . 'includes/class-wp-fdeal-product.php';

        require_once plugin_dir_path(dirname(__FILE__)) . 'admin/connector.php';
        require_once plugin_dir_path(dirname(__FILE__)) . 'admin/new_connector.php';


        $this->loader = new Wp_Fdeal_Loader();

    }

    /**
     * Define the locale for this plugin for internationalization.
     *
     * Uses the Wp_Fdeal_i18n class in order to set the domain and to register the hook
     * with WordPress.
     *
     * @since    1.0.0
     * @access   private
     */
    private function set_locale()
    {

        $plugin_i18n = new Wp_Fdeal_i18n();

        $this->loader->add_action('plugins_loaded', $plugin_i18n, 'load_plugin_textdomain');

    }

    /**
     * Register all of the hooks related to the admin area functionality
     * of the plugin.
     *
     * @since    1.0.0
     * @access   private
     */
    private function define_admin_hooks()
    {

        $plugin_admin = new Wp_Fdeal_Admin($this->get_plugin_name(), $this->get_version(),$this->actualPluginVersion('version'));

        $this->loader->add_action('admin_enqueue_scripts', $plugin_admin, 'enqueue_styles');
        $this->loader->add_action('admin_enqueue_scripts', $plugin_admin, 'enqueue_scripts');

        $this->loader->add_action('admin_menu', $plugin_admin, 'add_plugin_admin_menu');

        $this->loader->add_action('wp_ajax_merge_categories', $plugin_admin, 'merge_categories');
        $this->loader->add_action('wp_ajax_clear_merged_categories', $plugin_admin, 'clear_merged_categories');
        $this->loader->add_action('wp_ajax_add_job', $plugin_admin, 'add_job');
        $this->loader->add_action('wp_ajax_repeat_price_process', $plugin_admin, 'repeat_price_process');
        $this->loader->add_action('wp_ajax_get_jobs_status_by_array_id', $plugin_admin, 'get_jobs_status_by_array_id');
        $this->loader->add_action('wp_ajax_import_dealer_categories_tree', $plugin_admin, 'import_dealer_categories_tree');
        $this->loader->add_action('wp_ajax_cancel_command', $plugin_admin, 'cancel_command');
        $this->loader->add_action('wp_ajax_update_schedule_date', $plugin_admin, 'update_schedule_date');
        $this->loader->add_action('wp_ajax_update_dealer_params', $plugin_admin, 'update_dealer_params');
        $this->loader->add_action('wp_ajax_add_price_markup', $plugin_admin, 'add_price_markup');
        $this->loader->add_action('wp_ajax_repeat_price_markup', $plugin_admin, 'repeat_price_markup');
        $this->loader->add_action('wp_ajax_update_price_markup', $plugin_admin, 'update_price_markup');
        $this->loader->add_action('wp_ajax_delete_price_markup', $plugin_admin, 'delete_price_markup');
        $this->loader->add_action('wp_ajax_update_options_merge', $plugin_admin, 'update_options_merge');
        $this->loader->add_action('wp_ajax_get_list_options_merge', $plugin_admin, 'get_list_options_merge');
        $this->loader->add_action('wp_ajax_update_replaces_options_merge', $plugin_admin, 'update_replaces_options_merge');
        $this->loader->add_action('wp_ajax_add_options_merge', $plugin_admin, 'add_options_merge');
        $this->loader->add_action('wp_ajax_add_multi_options_merge_by_section_id', $plugin_admin, 'add_multi_options_merge_by_section_id');
        $this->loader->add_action('wp_ajax_delete_options_merge', $plugin_admin, 'delete_options_merge');
        $this->loader->add_action('wp_ajax_delete_empty_bindings', $plugin_admin, 'delete_empty_bindings');
        $this->loader->add_action('wp_ajax_job_checker', $plugin_admin, 'job_checker');


        $plugin_basename = plugin_basename(plugin_dir_path(__DIR__) . $this->plugin_name . '.php');
        $this->loader->add_filter('plugin_action_links_' . $plugin_basename, $plugin_admin, 'add_action_links');
        add_filter('cron_schedules', array($this, 'job_checker_add'));
        add_filter('cron_schedules', array($this, 'update_available_distributors'));


        // Schedule an action if it's not already scheduled
        if (!wp_next_scheduled('job_checker_add')) {
            wp_schedule_event(time(), 'every_minute_job_checker', 'job_checker_add');
        }
        if (!wp_next_scheduled('update_available_distributors')) {
            wp_schedule_event(time(), 'hourly', 'update_available_distributors');
        }

        $this->loader->add_action('job_checker_add', $plugin_admin, 'job_checker');
        $this->loader->add_action('update_available_distributors', $plugin_admin, 'update_available_distributors');

        //add_action('job_checker_add', array($this, 'job_checker'));


    }

    function job_checker_add($schedules)
    {
        $schedules['every_minute_job_checker'] = array('interval' => 60, 'display' => __('Every 60 Seconds', 'textdomain'));
        return $schedules;
    }

    function update_available_distributors($schedules)
    {
        $schedules['hourly'] = array('interval' => 3600, 'display' => __('Every 1 Hour', 'textdomain'));
        return $schedules;
    }


    /**
     * Register all of the hooks related to the public-facing functionality
     * of the plugin.
     *
     * @since    1.0.0
     * @access   private
     */
    private function define_public_hooks()
    {

        $plugin_public = new Wp_Fdeal_Public($this->get_plugin_name(), $this->get_version());

        $this->loader->add_action('wp_enqueue_scripts', $plugin_public, 'enqueue_styles');
        $this->loader->add_action('wp_enqueue_scripts', $plugin_public, 'enqueue_scripts');

        $this->loader->add_filter('get_the_terms', $this, 'exclude_tags');


    }

    function exclude_tags($tags)
    {

        $dealer_names = array();
        $get_dealer_names = $this->db->get_results("select price_name from {$this->db->prefix}4dealer_prices");
        if (!is_null($get_dealer_names) && !empty($get_dealer_names)) {
            foreach ($get_dealer_names as $dealer_name) {
                $dealer_names[] = $dealer_name->price_name;
            }

            $newtags = array();

            foreach ($tags as $tag) {
                if (current_user_can('activate_plugins'))
                    return $tags;
                if (!in_array($tag->name, $dealer_names))
                    $newtags[] = $tag;
            }
            return $newtags;
        }
        return $tags;
    }

    /*    function exclude_tag_links($tags)
        {
            $newtags=array();
            $exclude_tags = array('Marvel');
            if ( current_user_can('activate_plugins'))
                return $tags;

            foreach ($tags as $tag) {
                $allow=true;
                foreach($exclude_tags as $e_tag){
                    if(strpos($tags,$e_tag)){
                        $allow=false;
                        break;
                    }
                }
                if($allow) {
                    $newtags[] = $tag;
                }
            }
            return $newtags;
        }*/

    /**
     * Run the loader to execute all of the hooks with WordPress.
     *
     * @since    1.0.0
     */
    public function run()
    {
        // $this->loader->add_filter('the_tags',$this,'exclude_tag_links');
        $this->loader->run();

        /**
         * Проверка для процедуры обновления (миграции в структуре БД и т.д.)
         *
         * Начиная с версии 1.4.0 появилась процедура обновления,
         * для всех предыдущих версий флагом будет являться файл 'v140.txt'
         */

        /*if (str_replace('.', '', $this->version) > '140') {
            return;
        }*/
        
        $cVersion = (int) preg_replace('/\D/', '', $this->version);
        $vFile = plugin_dir_path(dirname(__FILE__)) . 'admin/v'.$cVersion.'.txt';

        if (!file_exists($vFile)) {
            if ($status = $this->runUpdateProcedure()) {
                file_put_contents($vFile, 'v'.$cVersion);
            }
        }
    }

    /**
     * The name of the plugin used to uniquely identify it within the context of
     * WordPress and to define internationalization functionality.
     *
     * @return    string    The name of the plugin.
     * @since     1.0.0
     */
    public function get_plugin_name()
    {
        return $this->plugin_name;
    }

    /**
     * The reference to the class that orchestrates the hooks with the plugin.
     *
     * @return    Wp_Fdeal_Loader    Orchestrates the hooks of the plugin.
     * @since     1.0.0
     */
    public function get_loader()
    {
        return $this->loader;
    }

    /**
     * Retrieve the version number of the plugin.
     *
     * @return    string    The version number of the plugin.
     * @since     1.0.0
     */
    public function get_version()
    {
        return $this->version;
    }

    /**
     * Remote check plugin version once per day
     * Based on checking create file time
     */
    function remoteUpdatePluginVersion()
    {
        $lastUpdate = @filemtime($this->localActualVersionFile);
        $remoteFile = 'https://4dealer.ru/files/wp_version.txt';

        if (date('D m Y') == date('D m Y', $lastUpdate))
            return;

        file_put_contents($this->localActualVersionFile, file_get_contents($remoteFile));
    }

    /**
     * Get actual version from local file on every page refresh
     */
    function actualPluginVersion()
    {
        return trim(file_get_contents($this->localActualVersionFile));
    }

    /**
     * Процедура внесения изменений в структуру плагина в связи с переходом на новую версию
     * выполняется только после перехода с предыдущей версии
     *
     * @return false|string
     *
     * @since    1.4.0
     */
    function runUpdateProcedure()
    {
        $status = true;
        
        // Updates for version 1.4.0
        $chk = $this->db->get_results("SHOW TABLES LIKE '{$this->db->prefix}4dealer_options_merge'");
        if (count($chk) == 0 ) {
            $charset_collate = $this->db->get_charset_collate();

            $this->db->query("DELETE FROM {$this->db->prefix}4dealer_categories_term WHERE term_id NOT IN (
                SELECT term_id FROM {$this->db->prefix}terms
            )");

            $this->db->query("ALTER TABLE {$this->db->prefix}4dealer_categories_term 
                       DROP FOREIGN KEY {$this->db->prefix}4dealer_categories_term_ibfk_1");

            $this->db->query("ALTER TABLE {$this->db->prefix}4dealer_categories_term ADD CONSTRAINT {$this->db->prefix}4dealer_categories_term_ibfk_1 
                       FOREIGN KEY (term_id) REFERENCES {$this->db->prefix}terms(term_id) ON DELETE CASCADE ON UPDATE CASCADE");

            $this->db->query("ALTER TABLE {$this->db->prefix}4dealer_price_markup DROP PRIMARY KEY");
            $this->db->query("ALTER TABLE `{$this->db->prefix}4dealer_price_markup` ADD `id` INT NOT NULL AUTO_INCREMENT FIRST, ADD PRIMARY KEY  (`id`)");

            /** Result of the last sql operation must be a status */
            $status = $this->db->query("CREATE TABLE IF NOT EXISTS {$this->db->prefix}4dealer_options_merge (
                `id` INT NOT NULL AUTO_INCREMENT,
                `4dealer_price_id` INT NOT NULL,
                `section_id` INT NOT NULL,
                `option_name` VARCHAR(255) NOT NULL,
                `changed_option_name` VARCHAR(255) NOT NULL,
                `replaces` TEXT NOT NULL,
                PRIMARY KEY (`id`),
                INDEX (`4dealer_price_id`)                    
            ) $charset_collate ENGINE=InnoDB");
        }
        if (!$status) {
            return false;
        }
        
        // Updates for version 1.4.1
        $chk = $this->db->get_results("SHOW COLUMNS FROM `{$this->db->prefix}4dealer_price_markup` LIKE 'price_is_rrc'");
        if (count($chk) == 0 ) {
            /** Result of the last sql operation must be a status */
            $status = $this->db->query("ALTER TABLE {$this->db->prefix}4dealer_price_markup ADD `price_is_rrc` INT NOT NULL AFTER `markup`");
        }

        if (!$status) {
            return false;
        }

        // Updates for version 1.4.2
        $chk = $this->db->get_results("SHOW COLUMNS FROM `{$this->db->prefix}4dealer_prices` LIKE 'round_price'");

        if (count($chk) == 0 ) {
            $this->db->query("ALTER TABLE {$this->db->prefix}4dealer_prices ADD `round_price` INT NOT NULL");
            $this->db->query("ALTER TABLE {$this->db->prefix}4dealer_prices ADD `round_rule` INT NOT NULL");
            $this->db->query("ALTER TABLE {$this->db->prefix}4dealer_prices ADD `auto_load_options` INT NOT NULL");

            /** Result of the last sql operation must be a status */
            $status = $this->db->query("ALTER TABLE {$this->db->prefix}4dealer_prices ADD `product_add_table` INT NOT NULL");
        }
        
        // Updates for version 1.5.1
        $chk = $this->db->get_results("SHOW COLUMNS FROM `{$this->db->prefix}4dealer_prices` LIKE 'public_options'");
        if (count($chk) == 0 ) {
            $status = $this->db->query("ALTER TABLE {$this->db->prefix}4dealer_prices ADD `public_options` INT(1) NOT NULL");
        }
        
        return $status;
    }

}
